from runtests.mpi import MPITest

@MPITest(commsize=[1, 2])
def test_uncollective_error(comm):
    if comm.rank == 1:
        raise ValueError("This is a dummy error. The test will fail. but shall not hang")



"""
    Finding reference cycles of objects.

    The analysis is more or less static and oriented for
    single thread applications. Use objgraph for some of 
    the lower level operations.

    main entry point is:

    .. code::

        assert_no_backcycles(*objs)

    This is not very powerful, as it works against the
    intuition: if objs references a cycle it won't find it.
    if only finds cycles that include objs.

    .. code::

        assert_no_cycles(*objs)

    will find cycles that started from objs, but there are
    in general too many false positives to make this of any
    practical usefulness.

"""

import gc

def assert_no_cycles(*objs):
    # FIXME: this needs culling -- but how?

    gc.collect()
    sccs = tarjan(objs, get_referrers=gc.get_referents)

    if len(sccs) > 0:
        show_cycles(sccs)

    assert len(sccs) == 0

def assert_no_backcycles(*objs):
    """ Assert no objects on the list induces any cycles
        in the back reference list.

        e.g. 

        .. code::

            a = 3O

            assert_no_backcycles(a)

            a = []
            b = [a]
            a[0] = b

            assert_no_backcycles(a)
    """
    gc.collect()
    sccs = tarjan(objs, get_referrers=gc.get_referrers)

    if len(sccs) > 0:
        show_cycles(sccs)

    assert len(sccs) == 0

def show_cycles(sccs, joined=False):
    import objgraph
    a = sccs
    if joined:
        a = []
        for scc in sccs:
            a.extend(scc)
        a = [a]

    for scc in a:
        objs = objgraph.at_addrs(scc)
        print(objgraph.typestats(objs))
        objgraph.show_backrefs(objs, max_depth=len(scc) + 5,
            filter=lambda x: id(x) in scc)

def isin(obj, l):
    # can not use 'in' because it checks for equality not identity.
    for x in l:
        if x is obj: return True
    return False

def ignore_frames(x):
    import inspect
    import types

    l = []
    if inspect.isclass(x):
        # must be a class object
        l.extend([x.__mro__, x.__dict__])

        if hasattr(x, '__weakref__'):
            l.extend([x.__weakref__])

        for member in x.__dict__.values():
            # ignore attributes.
            if inspect.isgetsetdescriptor(member):
                l.append(member)

    # ignore the module and module dict
    if inspect.ismodule(x):
        l.extend([x, x.__dict__])

    # ignore a frame; this will not work with multi-threaded applications
    # use refcycle in that case for live applications
    if inspect.isframe(x):
        # this can't detect multi-threaded.
        l.extend([x])

    return l

def tarjan(objs, get_referrers=gc.get_referrers,
        ignore=ignore_frames,
        getid=id,
        squeeze=True):
    """ Identifying strongly connected components from a directional graph.

        Algorithm is from

            https://en.wikipedia.org/wiki/Tarjan%27s_strongly_connected_components_algorithm

        Parameters
        ----------
        objs : list
            a list of objects to start the algorithm. The input graph consists of
            all objects connected to these objects by the get_referrers function.

        get_referrers: func(*objs)
            returns the neighbour of objects. This function represents the egdes and
            serves as the discovery function for vertices.

        squeeze : bool
            True, remove single item components except self-loops.

        getid : func(x)
            generates a unique id for the given object. ids are used to track objects

    """
    gindex = [0]
    index = {}
    lowlink = {}
    onStack = {}
    S = []

    id_to_obj = {}

    edges = {}

    # first traverse to obtain the full object list V,
    # and the id to object mapping,

    def bfs_action(x):
        id_to_obj[getid(x)] = x

    V = _bfs(objs,
            get_referrers,
            ignore=lambda x: ignore(x) + [id_to_obj],
            action=lambda x: id_to_obj.update({getid(x) : x}),
            getid=getid)

    # shrink id_to_obj to the same size as V, removing undesired objects
    id_to_obj = {k: id_to_obj[k] for k in V }

    #print('V', V)
    # initially, nothing is on the stack
    for v in V: onStack[v] = False

    def strongly_connect(v):
        sccs = []

        index[v] = gindex[0]
        lowlink[v] = gindex[0]
        gindex[0]  = gindex[0] + 1

        S.append(v)

        onStack[v] = True
        isloop = False

        W = []
        for w in _ignore_filter(get_referrers(id_to_obj[v]),
                          ignore=lambda x: [x] if getid(x) not in V else [],
                          extraids=set()
                          ):
            W.append(getid(w))

        for w in W:
            if w not in index:
                sccs.extend(strongly_connect(w))
                lowlink[v] = min(lowlink[v], lowlink[w])

            elif onStack[w]:
                lowlink[v] = min(lowlink[v], index[w])
                if v == w:
                    isloop = True

        if lowlink[v] == index[v]:
            # start a new strongly connected component
            scc = []
            while True:
                w = S.pop()
                onStack[w] = False
                # add w to the current strongly connected component
                scc.append(w)
                if w == v:
                    break

            # if the scc is singular and not
            # forming a loop, skip it.
            if len(scc) > 1 or isloop:
                # output
                sccs.append(scc)

        return sccs

    sccs = []
    for v in V:
        if v not in index:
            sccs.extend(strongly_connect(v))

    return sorted(sccs, key=lambda x:-len(x))

def _ignore_filter(referrers, ignore, extraids, getid=id):
    """ Ignore objects on the referrers list if ignore(x) is true or if x is in extra """
    r = []
    for ref in referrers:
        if ignore is not None:
            extraids.update(set([getid(o) for o in ignore(ref)]))

        if getid(ref) in extraids: continue

        r.append(ref)

    return r

def _bfs(objs, get_referrers, ignore=None, action=None, getid=id):
    """ A breadth first search traverse of the graph.
    """
    import types
    visited = set()
    referrers = list(objs)
    extraids = set()

    extraids.add(getid(objs))
    while True:
        front = []
        for ref in referrers:

            refid = getid(ref)

            if refid in visited:
                # already visited
                pass
            else:
                if action: action(ref)
                visited.add(refid)
                front.append(ref)

        if len(front) == 0:
            break

        extraids.add(getid(referrers))
        extraids.add(getid(front))

        newreferrers = get_referrers(*front)

        extraids.add(getid(newreferrers))

        referrers = _ignore_filter(newreferrers,
                ignore=ignore,
                extraids=extraids)

        #print(extraids)
        #print('referrers', [type(o) for o in referrers])
        #pprint(referrers)
        #input()
    return visited - extraids

def f():
    pass

class d:
    def __init__(self):
        pass

    def method(self):
        pass

    m2 = f

e = d()

f.e = e

def main():
    a1 = dict()
    a2 = dict()
    a3 = dict()
    a1['a2'] = a2
    a2['a3'] = a3
    a3['a1'] = a1
    b = dict()
    b['b'] = b
    c = dict()
    c['c'] = 'c'

    import gc
    import types
    print(len(
        _bfs([b],
            gc.get_referrers
            )
     ))

    sccs = tarjan([a1, b, c], gc.get_referrers)

    show_cycles(sccs, joined=True)
    print(sccs)
    del sccs
    gc.collect()


    sccs = tarjan([d, e, f])

    show_cycles(sccs, joined=True)

    return

    sccs = tarjan(gc.get_objects(), gc.get_referrers)
    print([len(i) for i in sccs])
    import objgraph
    objs = objgraph.at_addrs(sccs[0])
    print(objgraph.typestats(objs))

if __name__ == "__main__":
    main()



from runtests.mpi import MPITest, nompi

@MPITest(commsize=[1, 2])
def test_assert_block(comm):
    error = None
    try:
        with nompi(comm):
            if comm.rank == 0:
                raise ValueError("This is an error raised on rank 0. Other ranks will see a RuntimeError instead")
    except RuntimeError as e:
        assert comm.rank != 0
    except ValueError as e:
        assert comm.rank == 0
        



import time
from contextlib import contextmanager
import platform
import time
import os
from collections import defaultdict
import itertools
import json

def get_machine_info():
    """
    Return information about the machine, including host,
    system, and the python version.
    """
    return {'host':platform.node(),
            'system': platform.system(),
            'python_version': ".".join(platform.python_version_tuple())}


class BenchmarkLogger(object):
    """
    A class to serve as a session-wide benchmarking fixture, tracking
    the benchmark results for individual tests.

    A single JSON file is written to the specified output directory
    for each test. Machine info and related system information is also
    written to the JSON file.

    Parameters
    ----------
    output_dir : str
        the output directory to write results to
    comm : MPI comm, None, optional
        the MPI communicator or None if running serially
    version : str, optional
        the version of the source code being run
    git_hash : str, optional
        the short version of the git commit hash of the source code
    """
    def __init__(self, output_dir, comm=None, version=None, git_hash=None):

        # the header
        self.header = {}
        self.header['source_version'] = version
        self.header['git_hash'] = git_hash
        self.header.update(get_machine_info())
        self.header["datetime"] = time.strftime("%Y-%m-%d %H:%M:%S", time.gmtime())
        self.header['commsize'] = 1 if comm is None else comm.size

        self.comm = comm
        self.benchmarks = defaultdict(dict)
        self.tests_counter = defaultdict(int)

        # handle output dir
        self.output_dir = output_dir
        if self.comm is None or self.comm.rank == 0:
            if not os.path.exists(output_dir):
                os.makedirs(output_dir)

    def add_benchmark(self, result):
        """
        Add a benchmark result to the total set of benchmarks.

        Parameters
        ----------
        result : BenchmarkTimer
            an individual benchmark result
        """
        key = os.path.join(result.qualname, result.original_testname)
        name = key + '_%d' % self.tests_counter[key]

        # copy over the result
        r = result.benchmark.copy()
        r['attrs'] = result.attrs.copy()

        # add to total benchmarks
        self.benchmarks[name].update(r)

        # increment test counter
        self.tests_counter[key] += 1

    def report(self):
        """
        Report the benchmark results for all tests run.

        Benchmark results for individual files are stored in different
        files in :attr:`output_dir`. Results for variants of the same test
        function (via parametrization) are stored in the same file.

        Benchmark results for a single test variant are stored as a list,
        equal to the length of the comm size.

        .. note::
            When using MPI, this should be collectively, as benchmark
            results are gathered from all ranks. Only the
        """
        # group results by the file they will be written to
        # NOTE: this name ignores the parametrization (parametrized results
        # get written to same file)
        keyfunc = lambda x: self.benchmarks[x]['filename']
        groups = itertools.groupby(self.benchmarks.keys(), key=keyfunc)

        # sort groups so we avoid MPI issues
        groups = [(key, list(subgroup)) for key, subgroup in groups]
        groups = sorted(groups, key=lambda x: x[0])

        # loop over each parametrized test function
        for filename, subgroup in groups:

            # start with the info for this test
            result = {}
            result['config'] = self.header.copy()
            result['tests'] = []

            # loop over subgroups
            # NOTE: these are the parametrized test variants
            for i, key in enumerate(sorted(subgroup)):

                # extract the name of this test
                name = key.split('/')[-1]
                result['tests'].append(name)

                # initialize dict holding the individual results
                result[name] = {}

                # a group of benchmarks
                # NOTE: contains results for all tags within a single function run
                # for a parametrized variant
                benchmark_group = self.benchmarks[key]
                tags = sorted(benchmark_group['tags'])

                # for each tag, gather the benchmark results from each rank
                for tag in tags:
                    if self.comm is None:
                        benchmarks = [benchmark_group[tag]]
                    else:
                        benchmarks = self.comm.allgather(benchmark_group[tag])
                    result[name][tag] = benchmarks

                # store meta data
                result[name]['testname'] = benchmark_group['testname']
                result[name]['attrs'] = benchmark_group['attrs']

                # track section names for each test
                if i == 0:
                    result['tags'] = tags

            # write out
            if self.comm is None or self.comm.rank == 0:
                filename = os.path.join(self.output_dir, filename) + '.json'
                json.dump(result, open(filename, 'w'))

class BenchmarkTimer(object):
    """
    A class to serve as a function-scoped benchmarking fixture that is
    reponsible for timing each function.

    This will log its results with :class:`BenchmarkLogger`.

    Parameters
    ----------
    qualname : str
        the qualified name of the function being run; this is
        ``module_name . func_name``
    node :
        the request node corresponding to to this test function.
    """
    def __init__(self, qualname, node, comm=None):

        # add the testname
        self.qualname = qualname
        self.testname = node.name # NOTE: this will include parametrized ID

        # add the qualified test path for output file
        # (removes any parametrization IDs)
        self.original_testname = node.originalname
        if self.original_testname is None:
            self.original_testname = self.testname

        # the name of the file this function should be written too
        self.filename = '.'.join(self.qualname.split('.')[:-1] + [self.original_testname])

        self.comm = comm

        # store benchmarks here
        self.benchmark = {'filename':self.filename, 'testname':self.testname}
        self.benchmark['tags'] = []

        # store meta-data here
        self.attrs = {}

    @contextmanager
    def __call__(self, tag):
        """
        A context manager that uses :func:`time.time` to compute the elapsed
        time spent in the context.

        Parameters
        ----------
        tag : str
            an identifying tag to label this benchmark
        """
        if self.comm is not None:
            self.comm.barrier()

        start = time.time()
        yield
        end = time.time()

        # record the results in benchmarks attribute
        elapsed = end-start
        self.benchmark[tag] = elapsed
        self.benchmark['tags'].append(tag)



from ..tester import Tester as BaseTester

import pytest
import traceback
import warnings
import sys
import os
import contextlib
import time
from argparse import ArgumentParser
from contextlib import contextmanager

if sys.version_info[0] == 2:
    from cStringIO import StringIO
else:
    from io import StringIO
import re

def fix_titles(s):
    pattern = '=====+'
    return re.sub(pattern, lambda x: x.group(0).replace('=', '-'), s)

class Rotator(object):
    """ in a rotator every range runs in terms """
    def __init__(self, comm):
        self.comm = comm
    def __enter__(self):
        self.comm.Barrier()
        for i in range(self.comm.rank):
            self.comm.Barrier()
    def __exit__(self, type, value, tb):
        for i in range(self.comm.rank, self.comm.size):
            self.comm.Barrier()
        self.comm.Barrier()

@contextmanager
def nompi(comm):
    errored = False
    error = None
    try:
        yield
    except Exception as e:
        errored = True
        error = e
    finally:
        anyerrored = any(comm.allgather(errored))

    if anyerrored:
        if error is None:
            raise RuntimeError("Some ranks failed")
        else:
            raise error


communicators = {
}

class WorldTooSmall(Exception): pass

def create_comm(size):
    from mpi4py import MPI
    if MPI.COMM_WORLD.size < size:
        raise WorldTooSmall

    color = 0 if MPI.COMM_WORLD.rank < size else 1
    if size not in communicators:
        if MPI.COMM_WORLD.size == size:
            comm = MPI.COMM_WORLD
        elif size == 1:
            comm = MPI.COMM_SELF
        else:
            comm = MPI.COMM_WORLD.Split(color)
        communicators[size] = comm

    return communicators[size], color

def MPITestFixture(commsize, scope='function'):
    """ Create a test fixture for MPI Communicators of various commsizes """

    @pytest.fixture(params=commsize, scope=scope)
    def fixture(request):
        from mpi4py import MPI
        MPI.COMM_WORLD.barrier()
        try:
            comm, color = create_comm(request.param)

            if color != 0:
                pytest.skip("Not using communicator %d" %(request.param))
                return None
            else:
                return comm

        except WorldTooSmall:
            pytest.skip("Not using communicator %d" % request.param)
            return None

    return fixture

def MPITest(commsize):
    """
    A decorator that repeatedly calls the wrapped function,
    with communicators of varying sizes.

    This converts the test to a generator test; therefore the
    underlyig test shall not be a generator test.

    Parameters
    ----------
    commsize: scalar or tuple
        Sizes of communicator to use

    Usage
    -----
    @MPITest(commsize=[1, 2, 3])
    def test_stuff(comm):
        pass
    """
    from mpi4py import MPI
    if not isinstance(commsize, (tuple, list)):
        commsize = (commsize,)

    sizes = sorted(list(commsize))

    def dec(func):

        @pytest.mark.parametrize("size", sizes)
        def wrapped(size, *args):
            func_names = MPI.COMM_WORLD.allgather(func.__name__)
            if not all(func_names[0] == i for i in func_names):
                raise RuntimeError("function calls mismatched", func_names)

            try:
                comm, color = create_comm(size)
            except WorldTooSmall:
                return pytest.skip("Test skipped because world is too small. Include the test with mpirun -n %d" % (size))

            try:
                if color == 0:
                    rt = func(*args, comm=comm)
                if color == 1:
                    rt = None
                    #pytest.skip("rank %d not needed for comm of size %d" %(MPI.COMM_WORLD.rank, size))
            finally:
                MPI.COMM_WORLD.barrier()

            return rt
        wrapped.__name__ = func.__name__
        return wrapped
    return dec

def MPIWorld(NTask, required=1, optional=False):
    """
    A decorator that repeatedly calls the wrapped function,
    with communicators of varying sizes.

    .. note:: Deprecated
        See :func:`MPITest` instead

    Parameters
    ----------
    NTask : scalar or tuple
        Size of communicators ot use
    required : scalar or tuple
        Required sizes of communicators. If the MPI_WORLD is insufficient, a Error is raised
        to abort the tests.
    optional : boolean
        If requirement not satistied, skip the test.
    """
    warnings.warn("This function is deprecated, use MPITest instead.", DeprecationWarning)
    from mpi4py import MPI

    if not isinstance(NTask, (tuple, list)):
        NTask = (NTask,)

    if not isinstance(required, (tuple, list)):
        required = (required,)

    maxsize = max(required)
    if MPI.COMM_WORLD.size < maxsize and not optional:
        raise ValueError("Test Failed because the world is too small. Increase to mpirun -n %d, current size = %d" % (maxsize, MPI.COMM_WORLD.size))

    sizes = sorted(set(list(required) + list(NTask)))
    def dec(func):

        @pytest.mark.parametrize("size", sizes)
        def wrapped(size, *args):
            if MPI.COMM_WORLD.size < size:
                pytest.skip("Test skipped because world is too small. Include the test with mpirun -n %d" % (size))

            color = 0 if MPI.COMM_WORLD.rank < size else 1
            comm = MPI.COMM_WORLD.Split(color)

            if color == 0:
                rt = func(*args, comm=comm)

            MPI.COMM_WORLD.barrier()
            if color == 1:
                #pytest.skip("rank %d not needed for comm of size %d" %(MPI.COMM_WORLD.rank, size))
                rt = None
            comm.Free()
            return rt
        wrapped.__name__ = func.__name__
        return wrapped
    return dec

class Tester(BaseTester):
    """
    Run MPI-enabled tests using pytest, building the project first.

    Examples::
        $ python runtests.py my/module
        $ python runtests.py --single my/module
        $ python runtests.py my/module/tests/test_abc.py
        $ python runtests.py --mpirun="mpirun -np 4" my/module
        $ python runtests.py --mpirun="mpirun -np 4 xterm -hold -e"
        $ python runtests.py --mpirun="mpirun -np 4" --xterm my/module
    """

    @staticmethod
    def pytest_addoption(parser):
        """
        Add command-line options to specify MPI and coverage configuration
        """
        BaseTester.pytest_addoption(parser)

        parser.addoption("--mpirun", default="mpirun -n 4",
                help="Select MPI launcher, e.g. mpirun -n 4")

        parser.addoption("--xterm", default=False, action='store_true',
                help="If true, postfix 'xterm -hold -e' to mpirun.")

        parser.addoption("--single", default=False, action='store_true',
                help="Do not run via MPI launcher. ")

        parser.addoption("--mpisub", action="store_true", default=False,
                help="run process as a mpisub")

        parser.addoption("--mpisub-site-dir", default=None, help="site-dir in mpisub")


    def __init__(self, *args, **kwargs):
        """
        Parameters
        ----------
        package_file : str
            the path to the main directory of the source package
        module : str
            the name of the package to test
        extra path : list of str
            extra paths to include on PATH when building
        """
        super(Tester, self).__init__(*args, **kwargs)

    @property
    def comm(self):
        from mpi4py import MPI
        return MPI.COMM_WORLD

    def main(self, argv):
        # must bail after first dead test; avoiding a fault MPI collective state.
        argv.insert(1, '-x')

        config = self._get_pytest_config(argv)
        args = config.known_args_namespace

        # print help and exit
        if args.help:
            return config.hook.pytest_cmdline_main(config=config)

        # import project from system path
        args.pyargs = True

        # build / setup on the master
        if not args.mpisub:
            self._initialize_dirs(args)

            if not args.no_build:
                site_dir = self._do_build(args)

                if not args.bench:
                    # if we are here, we will run the tests, either as sub or single
                    # fix the path of the modules we are testing
                    config.args = self._fix_test_paths(site_dir, config.args)

            else:
                site_dir = None

            if args.build_only:
                sys.exit(0)

            if args.shell:
                self._do_shell(args, config)

            if not args.single:
                self._launch_mpisub(args, site_dir)

        else:

            capman = config.pluginmanager.getplugin('capturemanager')
            if capman:
                if hasattr(capman, 'suspend_global_capture'):
                    capman.suspend_global_capture()
                else:
                    capman.suspendcapture()

            # test on mpisub.
            if args.mpisub_site_dir:
                site_dir = args.mpisub_site_dir
                sys.path.insert(0, site_dir)
                os.environ['PYTHONPATH'] = site_dir

                if not args.bench:
                    # if we are here, we will run the tests, either as sub or single
                    # fix the path of the modules we are testing
                    config.args = self._fix_test_paths(site_dir, config.args)


        # extract the coverage-related options
        covargs = {}
        covargs['with_coverage'] = args.with_coverage
        covargs['config_file'] = args.cov_config
        covargs['html_cov'] = args.html_cov

        if args.mpisub:
            self._begin_capture(args)

        # run the tests
        try:
            code = None
            with self._run_from_testdir(args):
                code = self._test(config, comm=self.comm, **covargs)

        except:
            if args.mpisub:
                self._sleep()
                self.oldstderr.write("Fatal Error on Rank %d\n" % self.comm.rank)
                self.oldstderr.write(traceback.format_exc())
                self.oldstderr.flush()
                self.comm.Abort(-1)
            else:
                traceback.print_exc()
                sys.exit(1)

        if args.mpisub:
            self._end_capture_and_exit(code)
        else:
            sys.exit(code)

    def _launch_mpisub(self, args, site_dir):

        # extract the mpirun run argument
        parser = ArgumentParser(add_help=False)
        # these values are ignored. This is a hack to filter out unused argv.
        parser.add_argument("--single", default=False, action='store_true')
        parser.add_argument("--mpirun", default=None)
        parser.add_argument("--xterm", default=False, action='store_true')
        _args, additional = parser.parse_known_args()

        # now call with mpirun
        mpirun = args.mpirun.split()
        if args.xterm:
            mpirun.extend(['xterm', '-hold', '-e'])

        cmdargs = [sys.executable, '-u', sys.argv[0], '--mpisub']

        if site_dir is not None:
            # mpi subs will use system version of package
            cmdargs.extend(['--mpisub-site-dir=' + site_dir])

        # workaround the strict openmpi oversubscribe policy
        # the parameter is found from
        # https://github.com/open-mpi/ompi/blob/ba47f738871ff06b8e8f34b8e18282b9fe479586/orte/mca/rmaps/base/rmaps_base_frame.c#L169
        # see the faq:
        #   https://www.open-mpi.org/faq/?category=running#oversubscribing
        os.environ['OMPI_MCA_rmaps_base_oversubscribe'] = '1'
        os.environ['OMPI_MCA_rmaps_base_no_oversubscribe'] = '0'
        os.environ['OMPI_MCA_mpi_yield_when_idle'] = '1'

        os.execvp(mpirun[0], mpirun + cmdargs + additional)

        # if we are here os.execvp has failed; bail
        sys.exit(1)

    def _sleep(self):
        time.sleep(0.04 * self.comm.rank)

    def _begin_capture(self, args):
        self.oldstdout = sys.stdout
        self.oldstderr = sys.stderr
        self.newstdout = StringIO()
        self.newstderr = StringIO()

        if args.capture != 'no':
            if self.comm.rank != 0:
                sys.stdout = self.newstdout
                sys.stderr = self.newstderr

    def _end_capture_and_exit(self, code):
        if code != 0:
            # if any rank has a failure, print the error and abort the world.
            self._sleep()
            if self.comm.rank != 0:
                self.oldstderr.write("Test Failure due to rank %d\n" % self.comm.rank)
                self.oldstderr.write(self.newstdout.getvalue())
                self.oldstderr.write(self.newstderr.getvalue())
                self.oldstderr.flush()
            self.comm.Abort(-1)

        self.comm.barrier()
        with Rotator(self.comm):
            if self.comm.rank != 0:
                self.oldstderr.write("\n")
                self.oldstderr.write("=" * 32 + " Rank %d / %d " % (self.comm.rank, self.comm.size) + "=" * 32)
                self.oldstderr.write("\n")
                self.oldstderr.write(fix_titles(self.newstdout.getvalue()))
                self.oldstderr.write(fix_titles(self.newstderr.getvalue()))
                self.oldstderr.flush()

        sys.exit(0)

    @contextlib.contextmanager
    def _run_from_testdir(self, args):
        if not args.mpisub:
            with super(Tester, self)._run_from_testdir(args):
                yield
                return
        cwd = os.getcwd()

        try:
            assert(os.path.exists(self.TEST_DIR))
            self.comm.barrier()
            os.chdir(self.TEST_DIR)
            yield
        finally:
            os.chdir(cwd)



import pytest
import time

def test_benchmark1(benchmark):
    with benchmark("tag A"):
        time.sleep(1)

@pytest.mark.parametrize('x', [1, 2])
def test_benchmark2(benchmark, x):
    with benchmark("tag A"):
        time.sleep(1)

    with benchmark("tag B"):
        time.sleep(1)

    # store some meta-data about this run
    benchmark.attrs.update(x=x)



from setuptools import setup

def find_version(path):
    import re
    # path shall be a plain ascii text file.
    s = open(path, 'rt').read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              s, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Version not found")

setup(
    name="runtests",
    version=find_version("runtests/version.py"),
    author="Yu Feng, Nick Hand",
    author_email="rainwoodman@gmail.com",
    url="http://github.com/rainwoodman/runtests",
    description="Simple testing of fresh package builds using pytest, with optional mpi4py support",
    zip_safe = False,
    package_dir = {'runtests': 'runtests'},
    install_requires=['pytest', 'coverage'],
    license='BSD-2-Clause',
    packages= ['runtests', 'runtests.mpi'],
    requires=['pytest', 'coverage'],
    package_data = {'runtests' : ['tests/*.py', 'mpi/tests/*.py']},
    extras_require={'full':['mpi4py'], 'mpi':['mpi4py']}
)



from runtests.mpi import MPITest

@MPITest(commsize=[1, 2])
def test_mpicomm(comm):
    assert comm is not None

from runtests.mpi import MPITestFixture

comm1 = MPITestFixture(commsize=[1, 2])
def test_mpicomm(comm1):
    assert comm1 is not None

from runtests.mpi import MPIWorld
# this is deprecated
@MPIWorld(NTask=[1, 2], required=(1, 2))
def test_mpiworld(comm):
    assert comm is not None




from runtests.mpi import MPITest
import pytest
import time

def test_benchmark1(benchmark):

    comm = benchmark.comm
    with benchmark("test 1"):
        time.sleep((1+comm.rank)*0.25)

@pytest.mark.parametrize('x', [1, 2])
def test_benchmark2(benchmark, x):

    comm = benchmark.comm
    with benchmark("test 2"):
        time.sleep((1+comm.rank)*0.25)
